/**
 * \file rvc/uiformobjectlist.h
 * \brief RVC::UI::FORM_OBJECT_LIST class definitions
 *
 * \if NODOC
 * $Id: uiformobjectlist.h_v 1.13 2014/07/14 19:32:57 mju Exp $
 *
 * $Log: uiformobjectlist.h_v $
 * Revision 1.13  2014/07/14 19:32:57  mju
 * Add setFilter.
 *
 * Revision 1.12  2013/06/24 14:53:29  mju
 * In onBtnRemove handle multiple selection.
 * Don't force item to be selected in multi-selection mode.
 *
 * Revision 1.11  2013/06/21 21:38:51  mju
 * Add formstyle to allow multi-selection.
 * Remove virtual methods as only need delegates.
 *
 * Revision 1.10  2013/05/09 15:27:39  mju
 * Don't use const for filter pointer.
 *
 * Revision 1.9  2013/05/09 15:07:14  mju
 * Add create method with pointer to filter to take ownership.
 *
 * Revision 1.8  2013/05/09 14:51:55  mju
 * Support mgui::dlg_getobject_filter and deprecate use of rvc::getobjfilter.
 *
 * Revision 1.7  2012/07/27 14:03:25  mju
 * Add getObjTypeSet.
 *
 * Revision 1.4  2007/05/16 21:38:23  scowan
 * Added setObjectList method that only works before calling create method.
 *
 * Revision 1.1  2006/07/21 21:19:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMOBJECTLIST_H
#define	INC_RVC_UIFORMOBJECTLIST_H

#ifndef	INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

#ifndef	INC_MGUI_DLGGETOBJECTSTYLE_H
#include <mgui/dlggetobjectstyle.h>
#endif

#ifndef	INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
#include <rvc/objitemlist.h>
#endif

#ifndef	INC_RVC_OTYPESET_H
#include <rvc/otypeset.h>
#endif

#ifndef	INC_RVC_GETOBJFLAGS_H
#include <rvc/getobjflags.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class GETOBJFILTER;
	}
#endif

namespace RVC {
namespace UI {
	
//===================================================================================================================

//! Form containing basic object selection list with "add", "remove", and "remove all" buttons.
class FORM_OBJECT_LIST : public MGUI::FORM_COMPOSITE {
	public:

		enum FORMSTYLE {
			FORMSTYLE_ListSelectSingle =		0x00,
			FORMSTYLE_ListSelectMulti =		0x01,
			FORMSTYLE_ListSelectExtended =	0x02,
			};
	
		//! Constructor.
		FORM_OBJECT_LIST ();

		//! Destructor.
		virtual ~FORM_OBJECT_LIST ();
			
		//! Clears out all of the selected objects
		void Clear (
			bool notify							//!< Send notification
			);

		//! Create form with filter reference.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const MGUI::DLG_GETOBJECT_FILTER& filter,		//!< Object selection filter reference (MUST REMAIN VALID)
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			FORMSTYLE FormStyle = FORMSTYLE_ListSelectSingle,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create form and take ownership of filter.
		//! The form will take ownership of the specified filter pointer and the value will be returned as NULL.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,			//!< Object selection filter pointer (must not be NULL)
			MGUI::DLG_GETOBJECT_STYLE DlgStyle = MGUI::DLG_GETOBJECT_STYLE_OnlyExisting,	//!< Flags for DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			FORMSTYLE FormStyle = FORMSTYLE_ListSelectSingle,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create form with legacy filter.
		DEPRECATED ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,				//!< Parent pane
			const MISTRING& promptstr,							//!< Prompt string to use with DlgGetObjects
			const RVC::OBJTYPESET& ObjTypeSet,				//!< Set of object types that are selectable
			GETOBJFLAGS GetObjFlags = GETOBJFLAG_None,	//!< Flags for DlgGetObjects
			RVC::GETOBJFILTER* GetObjFilter = 0,			//!< Filter for DlgGetObjects
			int ListHeight = 5,									//!< Initial list box height
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		MGUI::LAYOUT_PANE& GetBtnPane ()
			{ return (m_BtnPane); }

		//! Get the current (highlighted) object.
		//! Will return an empty OBJITEM if there is nothing in the list or if list contains more than
		//! one item and there is not exactly one item highlighted.
		const RVC::OBJITEM& GetCurrentObject () const;
			
		//! Get the list of objects in the list			
		const RVC::OBJITEMLIST& GetObjectList () const
			{ return (m_ObjItemList); }

		//! Get set of allowed object types.
		const RVC::OBJTYPESET& GetObjTypeSet () const;

		//! Get list of objects highlighted.
		void GetSelectedObjectList (
			RVC::OBJITEMLIST& ObjItemList						//!< List returned, will be empty if none highlighted
			) const;

		//! Is the list empty?			
		bool IsEmpty () const
			{ return (m_ObjItemList.IsEmpty()); }

		//! Select objects via form.
		//! Behaves same as when 'Select' button pressed.
		ERRVALUE Select (
			bool notify = true						//!< Send notification if list changed
			);
			
		//! Set delegate to call for any change to objects in list.
		void SetDelegateOnListChanged (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnListChanged = delegate; }

		//! Set delegate to call when items highlighted in list.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate		//!< Delegate
			) { m_DelegateOnListSelection = delegate; }

		//! Set delegate to call for each object added to list.
		void SetDelegateOnObjectAdded (
			FastDelegate<void(const RVC::OBJITEM&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnObjectAdded = delegate; }

		//! Set delegate to call when single object removed from list.
		void SetDelegateOnObjectRemoved (
			FastDelegate<void(INT32)> delegate								//!< Delegate parameter: item index in list
			) { m_DelegateOnObjectRemoved = delegate; }

		//! Set delegate to call when single object selected (highlighted) in list.
		DEPRECATED_MSG("Use SetDelegateOnSelection") void SetDelegateOnObjectSelected (
			FastDelegate<void(const RVC::OBJITEM&, INT32)> delegate	//!< Delegate parameters: object item, item index in list
			) { m_DelegateOnObjectSelected = delegate; }

		//! Set delegate to call when objects added to list.
		void SetDelegateOnObjectsAdded (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnObjectsAdded = delegate; }

		//! Set delegate to call when all objects removed from list.
		void SetDelegateOnObjectsAllRemoved (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnObjectsAllRemoved = delegate; }

		//! Set new filter.
		//! Form will take ownership of new filter and value will be returned as NULL. Any currently selected object not allowed
		//! by the new filter will be removed from the list. The additional parameters affect whether the user will be prompted
		//! for a new selection and/or whether to call the delegate to notify. If the user cancels the selection (or is not prompted)
		//! the existing selection will be cleared. If no object is currently selected, the user will not be prompted and the delegate
		//! will not be called.
		void SetFilter (
			MGUI::DLG_GETOBJECT_FILTER *& pFilter,			//!< Object selection filter pointer (must not be NULL)
			bool bSelectIfNoCurrentAllowed,					//!< Reselect if no current object is allowed
			bool bNotifyIfChanged								//!< Notify if selection list changed
			);

		//! Set the list of objects in the list
		//! Must be called before Create() method.
		void SetObjectList (
			const RVC::OBJITEMLIST& ObjItemList
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_OBJECT_LIST THISCLASS;

		static RVC::OBJITEM *s_pNullItem;

		DELEGATE_VOID_NOPARMS m_DelegateOnListChanged;
		DELEGATE_VOID_NOPARMS m_DelegateOnListSelection;
		FastDelegate<void(const RVC::OBJITEM&, INT32)> m_DelegateOnObjectAdded;
		FastDelegate<void(INT32)> m_DelegateOnObjectRemoved;
		FastDelegate<void(const RVC::OBJITEM&, INT32)> m_DelegateOnObjectSelected;
		DELEGATE_VOID_NOPARMS m_DelegateOnObjectsAdded;
		DELEGATE_VOID_NOPARMS m_DelegateOnObjectsAllRemoved;
		MGUI::CTRL_LISTBOX m_ListBox;
		MGUI::CTRL_PUSHBUTTON m_BtnSelect;
		MGUI::CTRL_PUSHBUTTON m_BtnRemove;
		MGUI::CTRL_PUSHBUTTON m_BtnRemoveAll;
		MGUI::LAYOUT_PANE m_BtnPane;
		RVC::OBJITEMLIST m_ObjItemList;
		MISTRING m_PromptStr;
		const MGUI::DLG_GETOBJECT_FILTER *m_pFilter;
		MGUI::DLG_GETOBJECT_STYLE m_DlgStyle;
		bool m_bOwnFilter;
		bool m_bAllowSelectMulti;

		void NotifyListChanged ();
		void NotifyObjectsAllRemoved ();
		void OnListSelection ();
		void OnBtnRemove ();
		void OnBtnRemoveAll ();
		void OnBtnSelect ();

		FORM_OBJECT_LIST (const FORM_OBJECT_LIST&);
		FORM_OBJECT_LIST& operator= (const FORM_OBJECT_LIST&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//===================================================================================================================

}		// End namespace UI
}		// End namespace RVC

#endif	// INC_RVC_UIFORMOBJECTLIST_H
